@@ -74,13 +74,11 @@ class HAProxyDriver(LoadBalancerDriver):
         output.append('    log global')
         output.append('    option dontlognull')
         output.append('    option redispatch')
-        output.append('    maxconn 2000')
+        output.append('    maxconn 50000')
         output.append('    retries 3')
-        output.append('    timeout connect 5000ms')
-        output.append('    timeout client 50000ms')
-        output.append('    timeout server 5000ms')
-
-        serv_num = 1
+        output.append('    timeout connect 30000ms')
+        output.append('    timeout client 30000ms')
+        output.append('    timeout server 30000ms')
 
         for proto in self._config:
             protocfg = self._config[proto]
@@ -104,18 +102,53 @@ class HAProxyDriver(LoadBalancerDriver):
             #------------------------
             # Backend configuration
             #------------------------
+
             output.append('backend %s-servers' % proto)
             output.append('    mode %s' % proto)
             output.append('    balance %s' % protocfg['algorithm'])
 
+            # default healthcheck if none specified
+            monitor = 'check inter 30s'
+
             # HTTP specific options for the backend
             if proto == 'http':
-                output.append('    cookie SERVERID rewrite')
+                output.append('    cookie SERVERID insert indirect')
+                output.append('    option httpclose')
+                output.append('    option forwardfor')
 
-            for (addr, port, weight) in protocfg['servers']:
-                output.append('    server server%d %s:%s weight %d' %
-                              (serv_num, addr, port, weight))
-                serv_num += 1
+                if 'monitor' in self._config[proto]:
+                    mon = self._config[proto]['monitor']
+                    if mon['type'] == 'http':
+                        output.append('    option httpchk GET %s' %
+                                      mon['path'])
+                    # our timeout will be connect + read time
+                    output.append('    timeout check %ds' % mon['timeout'])
+                    # intentionally set rise/fall to the same value
+                    monitor = "check inter %ds rise %d fall %d" % (
+                              mon['delay'], mon['attempts'], mon['attempts'])
+
+                for (node_id, addr, port, weight) in protocfg['servers']:
+                    output.append('    server id-%s %s:%s cookie id-%s '
+                                  'weight %d %s' %
+                                  (node_id, addr, port, node_id,
+                                   weight, monitor))
+
+            # TCP specific options for the backend
+            else:
+                if 'monitor' in self._config[proto]:
+                    mon = self._config[proto]['monitor']
+                    if mon['type'] == 'http':
+                        output.append('    option httpchk GET %s' %
+                                      mon['path'])
+                    # our timeout will be connect + read time
+                    output.append('    timeout check %ds' % mon['timeout'])
+                    # intentionally set rise/fall to the same value
+                    monitor = "check inter %ds rise %d fall %d" % (
+                              mon['delay'], mon['attempts'], mon['attempts'])
+
+                for (node_id, addr, port, weight) in protocfg['servers']:
+                    output.append('    server id-%s %s:%s weight %d %s' %
+                                  (node_id, addr, port, weight, monitor))
 
         return '\n'.join(output) + '\n'
 
@@ -227,7 +260,7 @@ class HAProxyDriver(LoadBalancerDriver):
         else:
             self._bind(proto, '0.0.0.0', port)
 
-    def add_server(self, protocol, host, port, weight=1):
+    def add_server(self, protocol, node_id, host, port, weight=1):
         proto = protocol.lower()
         if weight is None:
             weight = 1
@@ -242,7 +275,7 @@ class HAProxyDriver(LoadBalancerDriver):
 
         if 'servers' not in self._config[proto]:
             self._config[proto]['servers'] = []
-        self._config[proto]['servers'].append((host, port, weight))
+        self._config[proto]['servers'].append((node_id, host, port, weight))
 
     def set_algorithm(self, protocol, algo):
         proto = protocol.lower()
@@ -253,6 +286,48 @@ class HAProxyDriver(LoadBalancerDriver):
         else:
             raise Exception('Invalid algorithm: %s' % protocol)
 
+    def add_monitor(self, protocol, mtype, delay, timeout, attempts, path):
+        proto = protocol.lower()
+        if mtype.lower() not in ['connect', 'http']:
+            raise Exception('Invalid monitor type: %s' % mtype)
+
+        # default values
+        if delay is None:
+            delay = 30
+        if attempts is None:
+            attempts = 2
+        if timeout is None:
+            timeout = delay
+        if (path is None) or (len(path) == 0):
+            path = '/'
+
+        if path[0] != '/':
+            path = '/' + path
+
+        try:
+            delay = int(delay)
+        except ValueError:
+            raise Exception("Non-integer 'delay' value: '%s'" % delay)
+
+        try:
+            timeout = int(timeout)
+        except ValueError:
+            raise Exception("Non-integer 'timeout' value: '%s'" % timeout)
+
+        try:
+            attempts = int(attempts)
+        except ValueError:
+            raise Exception("Non-integer 'attempts' value: '%s'" % attempts)
+
+        if timeout > delay:
+            raise Exception("Timeout cannot be greater than delay")
+
+        self._config[proto]['monitor'] = {'type': mtype.lower(),
+                                          'delay': delay,
+                                          'timeout': timeout,
+                                          'attempts': attempts,
+                                          'path': path}
+
     def create(self):
         self.ossvc.write_config(self._config_to_string())
         self.ossvc.service_stop()
@@ -272,7 +347,7 @@ class HAProxyDriver(LoadBalancerDriver):
         # restart, otherwise the log file will be kept open and not reappear.
         self.ossvc.syslog_restart()
 
-    def get_stats(self, protocol):
+    def get_stats(self, protocol=None):
         return self.ossvc.get_stats(protocol)
 
     def archive(self, method, params):
